"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultCheckboxViewStyles = exports.CheckboxView = void 0;
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _reactNativePaper = require("react-native-paper");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function arePropsEqual(prevProps, nextProps) {
  return prevProps.value === nextProps.value && prevProps.text === nextProps.text;
}
const CheckboxView = exports.CheckboxView = /*#__PURE__*/_react.default.memo(_CheckboxView, arePropsEqual);
function _CheckboxView(props) {
  const {
    value,
    onValueChange,
    text,
    outermostParentViewStyle = defaultCheckboxViewStyles.mainView,
    checkboxParentViewStyle = defaultCheckboxViewStyles.checkboxView,
    textTouchableStyle,
    checkboxProps,
    textProps = {
      style: defaultCheckboxViewStyles.checkboxTextStyle,
      numberOfLines: 1,
      ellipsizeMode: "middle"
    }
  } = props;
  function customCheckboxValueTypeToRNPaperType(customCheckboxValueType) {
    return customCheckboxValueType === 'indeterminate' ? 'indeterminate' : customCheckboxValueType ? 'checked' : 'unchecked';
  }

  /**
   * This function modifies the change in value when the previous state was indeterminate.
   * If the prior value is 'indeterminate', it will mark the CheckBox as checked upon click.
   *
   * @param newValue This represents the updated CheckBox value after it's clicked.
   */
  function onValueChangeModifier() {
    // If the previous state was 'indeterminate', set checked to true
    if (value === 'indeterminate') onValueChange(true);else onValueChange(!value);
  }
  return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    style: outermostParentViewStyle
  }, /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    style: checkboxParentViewStyle
  }, /*#__PURE__*/_react.default.createElement(_reactNativePaper.Checkbox.Android, _extends({}, checkboxProps, {
    status: customCheckboxValueTypeToRNPaperType(value),
    onPress: onValueChangeModifier
  }))), text ? /*#__PURE__*/_react.default.createElement(_reactNative.TouchableOpacity, {
    style: textTouchableStyle,
    onPress: onValueChangeModifier
  }, /*#__PURE__*/_react.default.createElement(_reactNative.Text, textProps, text)) : null);
}
const defaultCheckboxViewStyles = exports.defaultCheckboxViewStyles = _reactNative.StyleSheet.create({
  mainView: {
    alignSelf: 'center',
    alignItems: 'center',
    flexDirection: 'row',
    marginEnd: 10
  },
  checkboxView: {
    marginStart: 5,
    transform: [{
      scale: 1.2
    }]
  },
  checkboxTextStyle: {
    color: "black",
    marginTop: _reactNative.Platform.OS === 'android' ? 2 : undefined
  }
});
//# sourceMappingURL=CheckboxView.js.map